@@ -10,6 +10,7 @@
 #include "flang/Common/idioms.h"
 #include "llvm/Support/raw_ostream.h"
 #include <algorithm>
+#include <set>
 #include <utility>
 
 namespace Fortran::parser {
@@ -159,6 +160,8 @@ const char &AllSources::operator[](Provenance at) const {
   return origin[origin.covers.MemberOffset(at)];
 }
 
+void AllSources::ClearSearchPath() { searchPath_.clear(); }
+
 void AllSources::AppendSearchPathDirectory(std::string directory) {
   // gfortran and ifort append to current path, PGI prepends
   searchPath_.push_back(directory);
@@ -221,26 +224,42 @@ ProvenanceRange AllSources::AddCompilerInsertion(std::string text) {
   return covers;
 }
 
+static void EmitPrefix(llvm::raw_ostream &o, llvm::raw_ostream::Colors color,
+    const std::string &prefix, bool showColors) {
+  if (prefix.empty()) {
+    return;
+  }
+  if (showColors) {
+    o.changeColor(color, true);
+  }
+  o << prefix;
+  if (showColors) {
+    o.resetColor();
+  }
+}
+
 void AllSources::EmitMessage(llvm::raw_ostream &o,
     const std::optional<ProvenanceRange> &range, const std::string &message,
+    const std::string &prefix, llvm::raw_ostream::Colors color,
     bool echoSourceLine) const {
   if (!range) {
+    EmitPrefix(o, color, prefix, this->getShowColors());
     o << message << '\n';
     return;
   }
   CHECK(IsValid(*range));
   const Origin &origin{MapToOrigin(range->start())};
-  std::visit(
+  common::visit(
       common::visitors{
           [&](const Inclusion &inc) {
-            o << inc.source.path();
             std::size_t offset{origin.covers.MemberOffset(range->start())};
-            SourcePosition pos{inc.source.FindOffsetLineAndColumn(offset)};
-            o << ':' << pos.line << ':' << pos.column;
-            o << ": " << message << '\n';
+            SourcePosition pos{inc.source.GetSourcePosition(offset)};
+            o << pos.path << ':' << pos.line << ':' << pos.column << ": ";
+            EmitPrefix(o, color, prefix, this->getShowColors());
+            o << message << '\n';
             if (echoSourceLine) {
               const char *text{inc.source.content().data() +
-                  inc.source.GetLineStartOffset(pos.line)};
+                  inc.source.GetLineStartOffset(pos.trueLineNumber)};
               o << "  ";
               for (const char *p{text}; *p != '\n'; ++p) {
                 o << *p;
@@ -255,7 +274,7 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
                 auto last{range->start() + range->size() - 1};
                 if (&MapToOrigin(last) == &origin) {
                   auto endOffset{origin.covers.MemberOffset(last)};
-                  auto endPos{inc.source.FindOffsetLineAndColumn(endOffset)};
+                  auto endPos{inc.source.GetSourcePosition(endOffset)};
                   if (pos.line == endPos.line) {
                     for (int j{pos.column}; j < endPos.column; ++j) {
                       o << '^';
@@ -267,14 +286,15 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
             }
             if (IsValid(origin.replaces)) {
               EmitMessage(o, origin.replaces,
-                  inc.isModule ? "used here"s : "included here"s,
+                  inc.isModule ? "used here"s : "included here"s, prefix, color,
                   echoSourceLine);
             }
           },
           [&](const Macro &mac) {
-            EmitMessage(o, origin.replaces, message, echoSourceLine);
             EmitMessage(
-                o, mac.definition, "in a macro defined here", echoSourceLine);
+                o, origin.replaces, message, prefix, color, echoSourceLine);
+            EmitMessage(o, mac.definition, "in a macro defined here", ""s,
+                color, echoSourceLine);
             if (echoSourceLine) {
               o << "that expanded to:\n  " << mac.expansion << "\n  ";
               for (std::size_t j{0};
@@ -284,7 +304,10 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
               o << "^\n";
             }
           },
-          [&](const CompilerInsertion &) { o << message << '\n'; },
+          [&](const CompilerInsertion &) {
+            EmitPrefix(o, color, prefix, this->getShowColors());
+            o << message << '\n';
+          },
       },
       origin.u);
 }
@@ -292,23 +315,24 @@ void AllSources::EmitMessage(llvm::raw_ostream &o,
 const SourceFile *AllSources::GetSourceFile(
     Provenance at, std::size_t *offset) const {
   const Origin &origin{MapToOrigin(at)};
-  return std::visit(common::visitors{
-                        [&](const Inclusion &inc) {
-                          if (offset) {
-                            *offset = origin.covers.MemberOffset(at);
-                          }
-                          return &inc.source;
-                        },
-                        [&](const Macro &) {
-                          return GetSourceFile(origin.replaces.start(), offset);
-                        },
-                        [offset](const CompilerInsertion &) {
-                          if (offset) {
-                            *offset = 0;
-                          }
-                          return static_cast<const SourceFile *>(nullptr);
-                        },
-                    },
+  return common::visit(common::visitors{
+                           [&](const Inclusion &inc) {
+                             if (offset) {
+                               *offset = origin.covers.MemberOffset(at);
+                             }
+                             return &inc.source;
+                           },
+                           [&](const Macro &) {
+                             return GetSourceFile(
+                                 origin.replaces.start(), offset);
+                           },
+                           [offset](const CompilerInsertion &) {
+                             if (offset) {
+                               *offset = 0;
+                             }
+                             return static_cast<const SourceFile *>(nullptr);
+                           },
+                       },
       origin.u);
 }
 
@@ -323,12 +347,20 @@ const char *AllSources::GetSource(ProvenanceRange range) const {
 std::optional<SourcePosition> AllSources::GetSourcePosition(
     Provenance prov) const {
   const Origin &origin{MapToOrigin(prov)};
-  if (const auto *inc{std::get_if<Inclusion>(&origin.u)}) {
-    std::size_t offset{origin.covers.MemberOffset(prov)};
-    return inc->source.FindOffsetLineAndColumn(offset);
-  } else {
-    return std::nullopt;
-  }
+  return common::visit(
+      common::visitors{
+          [&](const Inclusion &inc) -> std::optional<SourcePosition> {
+            std::size_t offset{origin.covers.MemberOffset(prov)};
+            return inc.source.GetSourcePosition(offset);
+          },
+          [&](const Macro &) {
+            return GetSourcePosition(origin.replaces.start());
+          },
+          [](const CompilerInsertion &) -> std::optional<SourcePosition> {
+            return std::nullopt;
+          },
+      },
+      origin.u);
 }
 
 std::optional<ProvenanceRange> AllSources::GetFirstFileProvenance() const {
@@ -341,14 +373,15 @@ std::optional<ProvenanceRange> AllSources::GetFirstFileProvenance() const {
 }
 
 std::string AllSources::GetPath(Provenance at) const {
-  const SourceFile *source{GetSourceFile(at)};
-  return source ? source->path() : ""s;
+  std::size_t offset{0};
+  const SourceFile *source{GetSourceFile(at, &offset)};
+  return source ? *source->GetSourcePosition(offset).path : ""s;
 }
 
 int AllSources::GetLineNumber(Provenance at) const {
   std::size_t offset{0};
   const SourceFile *source{GetSourceFile(at, &offset)};
-  return source ? source->FindOffsetLineAndColumn(offset).line : 0;
+  return source ? source->GetSourcePosition(offset).line : 0;
 }
 
 Provenance AllSources::CompilerInsertionProvenance(char ch) {
@@ -390,7 +423,7 @@ AllSources::Origin::Origin(ProvenanceRange r, const std::string &text)
     : u{CompilerInsertion{text}}, covers{r} {}
 
 const char &AllSources::Origin::operator[](std::size_t n) const {
-  return std::visit(
+  return common::visit(
       common::visitors{
           [n](const Inclusion &inc) -> const char & {
             return inc.source.content()[n];
@@ -425,11 +458,15 @@ std::optional<ProvenanceRange> CookedSource::GetProvenanceRange(
     return std::nullopt;
   }
   ProvenanceRange first{provenanceMap_.Map(cookedRange.begin() - &data_[0])};
-  if (cookedRange.size() <= first.size()) {
+  if (cookedRange.size() <= first.size()) { // always true when empty
     return first.Prefix(cookedRange.size());
   }
-  ProvenanceRange last{provenanceMap_.Map(cookedRange.end() - &data_[0])};
-  return {ProvenanceRange{first.start(), last.start() - first.start()}};
+  ProvenanceRange last{provenanceMap_.Map(cookedRange.end() - 1 - &data_[0])};
+  if (first.start() <= last.start()) {
+    return {ProvenanceRange{first.start(), last.start() - first.start() + 1}};
+  } else {
+    return std::nullopt;
+  }
 }
 
 std::optional<CharBlock> CookedSource::GetCharBlock(
@@ -493,27 +530,29 @@ llvm::raw_ostream &AllSources::Dump(llvm::raw_ostream &o) const {
   o << "AllSources range_ ";
   DumpRange(o, range_);
   o << '\n';
+  std::set<const SourceFile *> sources;
   for (const Origin &m : origin_) {
     o << "   ";
     DumpRange(o, m.covers);
     o << " -> ";
-    std::visit(common::visitors{
-                   [&](const Inclusion &inc) {
-                     if (inc.isModule) {
-                       o << "module ";
-                     }
-                     o << "file " << inc.source.path();
-                   },
-                   [&](const Macro &mac) { o << "macro " << mac.expansion; },
-                   [&](const CompilerInsertion &ins) {
-                     o << "compiler '" << ins.text << '\'';
-                     if (ins.text.length() == 1) {
-                       int ch = ins.text[0];
-                       o << "(0x";
-                       o.write_hex(ch & 0xff) << ")";
-                     }
-                   },
-               },
+    common::visit(common::visitors{
+                      [&](const Inclusion &inc) {
+                        if (inc.isModule) {
+                          o << "module ";
+                        }
+                        o << "file " << inc.source.path();
+                        sources.emplace(&inc.source);
+                      },
+                      [&](const Macro &mac) { o << "macro " << mac.expansion; },
+                      [&](const CompilerInsertion &ins) {
+                        o << "compiler '" << ins.text << '\'';
+                        if (ins.text.length() == 1) {
+                          int ch = ins.text[0];
+                          o << "(0x";
+                          o.write_hex(ch & 0xff) << ")";
+                        }
+                      },
+                  },
         m.u);
     if (IsValid(m.replaces)) {
       o << " replaces ";
@@ -521,6 +560,9 @@ llvm::raw_ostream &AllSources::Dump(llvm::raw_ostream &o) const {
     }
     o << '\n';
   }
+  for (const SourceFile *sf : sources) {
+    sf->Dump(o);
+  }
   return o;
 }
 
@@ -593,7 +635,7 @@ std::optional<CharBlock> AllCookedSources::GetCharBlock(
       return result;
     }
   }
-  return nullptr;
+  return std::nullopt;
 }
 
 void AllCookedSources::Dump(llvm::raw_ostream &o) const {
